--[[
Name: MiniButt-1.0
Revision: $Rev:$
Author(s): DarkImakuni
Credits: Gello
Website: http://www.wowace.com/
SVN: http://svn.wowace.com/root/trunk/MiniButtLib
Description: Minimap button creation library
Dependencies: AceLibrary
]]

local major = "MiniButt-1.0"
local minor = tonumber(string.sub("$Revision:$", 12, -3))

if not AceLibrary then error(major.." requires AceLibrary.") end
if not AceLibrary:IsNewVersion(major, minor) then return end

local _G = getfenv(0)

local MiniButt = {}
local MiniButt_mt = {}
setmetatable(MiniButt, MiniButt_mt)

local function print(text)
	ChatFrame1:AddMessage("|cffffff9a"..major.."|r: "..text)
end


--
-- API Functions
--


-- Dynamically create a button
--  id = name of the button. self.name is a good idea
--  settings = table where SavedVariables are stored for the button (optional)
--  init = table of default settings (optional)
--    .icon     = the icon you would like in your button
--    .left     = function called on left-clicking the button
--    .right    = function called on right-clicking the button
--    .text     = the text shown in the tooltip on mouseover
--    .drag     = the movement the button should follow when dragged ("SQUARE" or "CIRCLE")
--    .enabled  = shown or hidden on creation (1 or nil)
--    .position = degree angle to display the button on creation
--    .locked   = enable or disable movement (1 or nil)
--  returns the button if successful, nil if not
function MiniButt:Create(id, settings, init)
	self:argCheck(id, 2, "string")
	self:argCheck(settings, 3, "table", "nil")
	self:argCheck(init, 4, "table", "nil")

	if _G[id.."MinimapButton"] then
		self:error("A minimap button named `"..id.."' already exists")
		return nil
	end

	settings = settings or {}
	init = init or {}

	local name = id.."MinimapButton"

	local frame = CreateFrame("Button", name, Minimap)
	frame:SetWidth(31)
	frame:SetHeight(31)
	frame:SetFrameStrata("LOW")
	frame:SetToplevel(1)
	frame:SetHighlightTexture("Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight")
	frame:SetPoint("TOPLEFT", Minimap, "TOPLEFT")

	local icon = frame:CreateTexture(name.."Icon", "BACKGROUND")
	icon:SetTexture(init.icon or "Interface\\Icons\\INV_Misc_QuestionMark")
	icon:SetWidth(20)
	icon:SetHeight(20)
	icon:SetPoint("TOPLEFT", frame, "TOPLEFT", 7, -5)

	local overlay = frame:CreateTexture(name.."Overlay", "OVERLAY")
	overlay:SetTexture("Interface\\Minimap\\MiniMap-TrackingBorder")
	overlay:SetWidth(53)
	overlay:SetHeight(53)
	overlay:SetPoint("TOPLEFT", frame, "TOPLEFT")

	frame:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnClick", self.OnClick)
	frame:SetScript("OnMouseDown", function() _G[this:GetName().."Icon"]:SetTexCoord(0.1, 0.9, 0.1, 0.9) end)
	frame:SetScript("OnMouseUp", function() _G[this:GetName().."Icon"]:SetTexCoord(0, 1, 0, 1) end)
	frame:SetScript("OnEnter", self.OnEnter)
	frame:SetScript("OnLeave", function() GameTooltip:Hide() end)
	frame:SetScript("OnDragStart", self.OnDragStart)
	frame:SetScript("OnDragStop", self.OnDragStop)

	frame.tooltipTitle = id
	frame.leftClick = init.left or nil
	frame.rightClick = init.right or nil
	frame.tooltipText = init.tooltip or nil

	local first = 1

	for i in pairs(settings) do first = nil end -- settings has been populated before

	if first then -- define settings from init or default
		settings.drag = init.drag or "CIRCLE"
		settings.enabled = init.enabled or 1
		settings.position = init.position or self:GetDefaultPosition()
		settings.locked = init.locked or nil
	end

	frame.settings = settings
	table.insert(self.buttons, id)

	self:SetEnable(id, settings.enabled)

	return frame
end

-- Syntax sugar. AceLibrary("MiniButt-1.0")(self.name)
MiniButt_mt.__call = MiniButt.Create

-- Changes the icon of the button.
--  path = texture path, ie: "Interface\\AddOn\\MyMod\\MyModIcon" or nil
function MiniButt:SetIcon(id, path)
	self:argCheck(id, 2, "string")
	self:argCheck(path, 3, "string", "nil")

	local icon = _G[id.."MinimapButtonIcon"]

	if icon then
		if path then icon:SetTexture(path)
		else icon:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark") end
	else
		self:error("Cannot set icon to minimap button named `"..id.."', it does not exist")
	end
end

-- Sets the left-click function.
--  value = the desired function, ie: MyMod.OnMinimapLeftClick
function MiniButt:SetLeftClick(id, value)
	self:argCheck(id, 2, "string")
	self:argCheck(value, 3, "function", "nil")

	local button = _G[id.."MinimapButton"]

	if button then button.leftClick = value
	else
		self:error("Cannot set left-click function to minimap button named `"..id.."', it does not exist")
	end
end

-- Sets the right-click function.
--  value = the desired function, ie: MyMod.OnMinimapRightClick
function MiniButt:SetRightClick(id, value)
	self:argCheck(id, 2, "string")
	self:argCheck(value, 3, "function", "nil")

	local button = _G[id.."MinimapButton"]

	if button then button.rightClick = value
	else
		self:error("Cannot set right-click function to minimap button named `"..id.."', it does not exist")
	end
end

-- Sets the drag route.
--  value = "CIRCLE" or "SQUARE" (not case sensitive)
function MiniButt:SetDrag(id, value)
	self:argCheck(id, 2, "string")
	self:argCheck(value, 3, "string")

	local button = _G[id.."MinimapButton"]
	value = string.upper(value)

	if button then
		if value == "CIRCLE" or value == "SQUARE" then
			button.settings.drag = value
			self:Move(id)
		else
			self:error("Bad arguement #3 to `SetDrag' (`CIRCLE' or `SQUARE' expected, got `"..value.."')")
		end
	else
		self:error("Cannot set drag movement to minimap button named `"..id.."', it does not exist")
	end
end

-- Locks minimap button from moving
--  value = 1 or nil
function MiniButt:SetLock(id, value)
	self:argCheck(id, 2, "string")
	self:argCheck(value, 3, "number", "nil")

	local button = _G[id.."MinimapButton"]

	if button then button.settings.locked = value
	else
		self:error("Cannot set lock on minimap button named `"..id.."', it does not exist")
	end
end

-- Enables or disables the minimap button
--  value = 1 or nil
function MiniButt:SetEnable(id, value)
	self:argCheck(id, 2, "string")
	self:argCheck(value, 3, "number", "nil")

	local button = _G[id.."MinimapButton"]

	if button then
		button.settings.enabled = value

		if value then
			button:Show()
			MiniButt:Move(id, nil, 1)
		else
			button:Hide()
		end
	else
		self:error("Cannot enable or disable minimap button named `"..id.."', it does not exist")
	end
end

-- Returns a setting of this minimap button
--  setting = "LOCKED", "ENABLED", "DRAG" or "POSITION" (not case sensitive)
function MiniButt:GetSetting(id, setting)
	self:argCheck(id, 2, "string")
	self:argCheck(setting, 3, "string")

	setting = string.lower(setting)
	local button = _G[id.."MinimapButton"]

	if button then return button.settings[setting]
	else
		self:error("Cannot find settings for minimap button named `"..id.."', it does not exist")
	end
end

-- Sets the tooltip text.
-- No need to include your addon's name, it was registered as .tooltipTitle during :Create()
--  value = string (can include \n)
function MiniButt:SetTooltip(id, value)
	self:argCheck(id, 2, "string")
	self:argCheck(value, 3, "string", "nil")

	local button = _G[id.."MinimapButton"]

	if button then button.tooltipText = value
	else
		self:error("Cannot set tooltip text for minimap button named `"..id.."', it does not exist")
	end
end

-- Moves the button.
--  value = degree angle to display the button (optional)
--  force = force move irregardless of locked status (optional)
function MiniButt:Move(id, value, force)
	self:argCheck(id, 2, "string")
	self:argCheck(value, 3, "number", "string", "nil")

	value = tonumber(value) or nil
	local button = _G[id.."MinimapButton"]

	if button then
		if not button.settings.locked or force then
			local x, y
			local angle = value or button.settings.position

			if button.settings.drag == "SQUARE" then
				x, y = 110 * cos(angle), 110 * sin(angle)
				x, y = math.max(-82, math.min(x, 84)), math.max(-86, math.min(y, 82))
			else
				x, y = 80 * cos(angle), 80 * sin(angle)
			end

			button:SetPoint("TOPLEFT", "Minimap", "TOPLEFT", 54 - x, y - 54)
			button.settings.position = angle
		end
	else
		self:error("Cannot move minimap button named `"..id.."', it does not exist")
	end
end

-- Debug. Use no parameters to list all created buttons.
-- if id == nil then print a list of all created buttons, else print settings for selected id
function MiniButt:Debug(id)
	self:argCheck(id, 2, "string", "nil")

	if id then
		local button = _G[id.."MinimapButton"]

		if button then
			print("Settings for button named '"..id.."':")
			print(" positon = "..tostring(button.settings.position))
			print(" enabled = "..tostring(button.settings.enabled))
			print(" drag = "..tostring(button.settings.drag))
			print(" locked = "..tostring(button.settings.locked))

		else self:Debug(nil) end
	else
		local num = #self.buttons

		if num > 0 then
			print("Buttons created by MiniButt:")
			for i=1, num, 1 do print(" "..self.buttons[i]) end

		else print("Buttons created by MiniButt: nil") end
	end
end


--
-- Internal Functions
--

local function GetDefaultPosition()
	-- this gets a new default position by increments of 20 degrees
	local position, found = 0, 1

	while found do
		found = nil

		for i=1, #self.buttons do
			if _G[self.buttons[i].."MinimapButton"].settings.position == position then
				position = position + 20
				found = 1
			end
		end

		found = (position > 340) and 1 or found -- leave if we've done full circle
	end

	return position
end


local function OnEnter()
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	GameTooltip:AddLine(this.tooltipTitle, 1, 0.7, 0)

	if this.tooltipText then GameTooltip:AddLine(this.tooltipText, 0.8, 0.8, 0.8, 1, 1) end

	GameTooltip:Show()
end

local function OnDragStart()
	this:GetScript("OnMouseDown")()
	this:LockHighlight()
	this:SetScript("OnUpdate", self.OnUpdate)
end

local function OnDragStop()
	this:SetScript("OnUpdate", nil)
	this:UnlockHighlight()
	this:GetScript("OnMouseUp")()
end

local function OnUpdate()
	local xpos, ypos = GetCursorPosition()
	local xmin, ymin = Minimap:GetLeft(), Minimap:GetBottom()

	xpos = xmin - xpos / Minimap:GetEffectiveScale() + 70
	ypos = ypos / Minimap:GetEffectiveScale() - ymin - 70

	this.settings.position = math.deg(math.atan2(ypos, xpos))

	local id = string.gsub(this:GetName() or "", "MinimapButton$", "")
	MiniButt:Move(id)
end

local function OnClick()
	if arg1 == "LeftButton" and this.leftClick then this.leftClick()
	elseif arg1 == "RightButton" and this.rightClick then this.rightClick() end
end


--
-- Load this bitch!
--

local function activate(self, old, deactivate)
	if old then self.buttons = old.buttons
	else self.buttons = {} end

	if deactivate then deactivate(old) end
end

AceLibrary:Register(MiniButt, major, minor, activate)
